/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) Jan 2013, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "LICENSE.txt".
 *
 */

/*
 *
 *  mtcr.h - Mellanox Software tools (mst) driver definitions
 *
 */

#ifndef _MST_H
#define _MST_H


#ifdef __WIN__

#include<winsock2.h>
#include <windows.h>

#ifdef MTCR_EXPORTS
#define MTCR_API __declspec(dllexport)
#else
#define MTCR_API __declspec(dllimport)
#endif

typedef unsigned __int8  u_int8_t;
typedef __int8           int8_t;
typedef unsigned __int16 u_int16_t;
typedef __int16          int16_t;
typedef unsigned __int32 u_int32_t;
typedef __int32          int32_t;
typedef unsigned __int64 u_int64_t;
typedef __int64          int64_t;

#if defined(_WIN64)
    typedef __int64 MT_long_ptr_t;
    typedef unsigned __int64 MT_ulong_ptr_t;
#else
    typedef _W64 long MT_long_ptr_t;
    typedef _W64 unsigned long MT_ulong_ptr_t;
#endif

#elif defined(__DJGPP__)

	typedef unsigned char    u_int8_t;
	typedef char             int8_t;
	typedef unsigned short   u_int16_t;
	typedef short            int16_t;
	typedef unsigned int     u_int32_t;
	typedef long             int32_t;
	typedef unsigned long long u_int64_t;
	typedef long long        int64_t;

#define bswap_32(x) ntohl(x)
#define MTCR_API

#else  /* UNIX */

#include <sys/types.h>
#define MTCR_API

#endif


#ifdef __cplusplus
extern "C" {
#endif

//#ifndef USE_IB_MGT
typedef struct mib_private_t {
    int  dummy;
} MIB_Private;
//#else
//#include "mtcr_ib_private.h"
//#endif

typedef enum MError {
    ME_OK = 0,
    ME_ERROR,
    ME_BAD_PARAMS,
    ME_CR_ERROR,
    ME_NOT_IMPLEMENTED,
    ME_SEM_LOCKED,
    ME_MEM_ERROR,

    ME_MAD_SEND_FAILED,
    ME_UNKOWN_ACCESS_TYPE,
    ME_UNSUPPORTED_DEVICE,
    ME_REG_NOT_SUPPORTED,

    // errors regarding REG_ACCESS
    ME_REG_ACCESS_OK = 0,
    ME_REG_ACCESS_BAD_STATUS_ERR = 0x100,
    ME_REG_ACCESS_BAD_METHOD,
    ME_REG_ACCESS_NOT_SUPPORTED,
    ME_REG_ACCESS_DEV_BUSY,
    ME_REG_ACCESS_VER_NOT_SUPP,
    ME_REG_ACCESS_UNKNOWN_TLV,
    ME_REG_ACCESS_REG_NOT_SUPP,
    ME_REG_ACCESS_CLASS_NOT_SUPP,
    ME_REG_ACCESS_METHOD_NOT_SUPP,
    ME_REG_ACCESS_BAD_PARAM,
    ME_REG_ACCESS_RES_NOT_AVLBL,
    ME_REG_ACCESS_MSG_RECPT_ACK,
    ME_REG_ACCESS_UNKNOWN_ERR,
    ME_REG_ACCESS_SIZE_EXCCEEDS_LIMIT,
    ME_REG_ACCESS_CONF_CORRUPT,
    ME_REG_ACCESS_LEN_TOO_SMALL,
    ME_REG_ACCESS_BAD_CONFIG,
    ME_REG_ACCESS_ERASE_EXEEDED,
    ME_REG_ACCESS_INTERNAL_ERROR,

    // errors regarding ICMD
    ME_ICMD_STATUS_CR_FAIL = 0x200,       // cr-space access failure
    ME_ICMD_INVALID_OPCODE,
    ME_ICMD_INVALID_CMD,
    ME_ICMD_OPERATIONAL_ERROR,
    ME_ICMD_BAD_PARAM,
    ME_ICMD_BUSY,
    ME_ICMD_INIT_FAILED,
    ME_ICMD_NOT_SUPPORTED,
    ME_ICMD_STATUS_SEMAPHORE_TO,          // timed out while trying to take semaphore
    ME_ICMD_STATUS_EXECUTE_TO,            // timed out while waiting for command to execute
    ME_ICMD_STATUS_IFC_BUSY,
    ME_ICMD_STATUS_ICMD_NOT_READY,
    ME_ICMD_UNSUPPORTED_ICMD_VERSION,
    ME_ICMD_UNKNOWN_STATUS,
    ME_ICMD_ICM_NOT_AVAIL,
    ME_ICMD_WRITE_PROTECT,
    ME_ICMD_SIZE_EXCEEDS_LIMIT,

    //errors regarding Tools CMDIF
    ME_CMDIF_BUSY = 0x300,
    ME_CMDIF_TOUT,
    ME_CMDIF_BAD_STATUS,
    ME_CMDIF_BAD_OP,
    ME_CMDIF_NOT_SUPP,
    ME_CMDIF_BAD_SYS,
    ME_CMDIF_UNKN_TLV,
    ME_CMDIF_RES_STATE,
    ME_CMDIF_UNKN_STATUS,

    //errors regarding MAD IF
    ME_MAD_BUSY = 0x400,
    ME_MAD_REDIRECT,
    ME_MAD_BAD_VER,
    ME_MAD_METHOD_NOT_SUPP,
    ME_MAD_METHOD_ATTR_COMB_NOT_SUPP,
    ME_MAD_BAD_DATA,
    ME_MAD_GENERAL_ERR,

    ME_LAST
} MError;

// typedefs for UEFI
#ifdef UEFI_BUILD
#include <mft_uefi_common.h>
#endif
typedef enum MType_t {MST_FPGA    = 0x1,
                      MST_MLNXOS  = 0x2,
                      MST_LPC     = 0x4,
                      MST_PCI     = 0x8,
                      MST_PCICONF = 0x10,
                      /*MST_CALBR,*/
                      MST_USB     = 0x20,
                      MST_IB      = 0x40,
                      MST_IF      = 0x80,
                      MST_PPC     = 0x100,
                      MST_USB_DIMAX=0x200,
                      MST_FWCTX   = 0x400,
                      MST_REMOTE  = 0x800,
#ifdef ENABLE_MST_DEV_I2C
                      MST_DEV_I2C = 0x1000,
#endif
                     MST_DEFAULT = 0xffffffff & ~MST_FPGA
} MType;

typedef enum DType_t {MST_GAMLA, MST_TAVOR, MST_DIMM, MST_NOADDR} DType;
#define MST_ANAFA2 MST_TAVOR
#define MST_EEPROM MST_GAMLA
typedef enum Mdevs_t {
    MDEVS_GAMLA     = 0x01, /*  Each device that actually is a Gamla */
    MDEVS_I2CM      = 0x02, /*  Each device that can work as I2C master */
    MDEVS_MEM       = 0x04, /*  Each device that is a memory driver (vtop) */
    MDEVS_TAVOR_DDR = 0x08, /*  Each device that maps to DDR */
    MDEVS_TAVOR_UAR = 0x10, /*  Each device that maps to UAR */
    MDEVS_TAVOR_CR  = 0x20, /*  Each device that maps to CR */
    MDEVS_IF        = 0x40, /*  Standard device  interface */
    MDEVS_REM       = 0x80, /*  Remote devices */
    MDEVS_PPC       = 0x100, /*  PPC devices */
    MDEVS_DEV_I2C   = 0x200, /* Generic linux kernel i2c device */
    MDEVS_IB        = 0x400, /* Cr access over IB Mads */
    MDEVS_MLNX_OS   = 0x800, /* access by CmdIf in MlnxOS */
    MDEVS_FWCTX     = 0x900, /* access by func/context (like UEFI) */
    MDEVS_LPC       = 0x1000,/* Access LPC region */
    MDEVS_FPGA      = 0x2000,/* Access LPC region */
    MDEVS_TAVOR     = (MDEVS_TAVOR_DDR|MDEVS_TAVOR_UAR|MDEVS_TAVOR_CR),
    MDEVS_ALL       = 0xffffffff
} Mdevs;

typedef enum {
    MACCESS_REG_METHOD_GET = 1,
    MACCESS_REG_METHOD_SET = 2
} maccess_reg_method_t;

typedef enum {
    AS_ICMD = 3,
    AS_CR_SPACE = 2,
    AS_SEMAPHORE = 0xa
} address_space_t;

typedef struct dev_info_t
{
    Mdevs         type;
    char          dev_name[512];

    union {
        struct {
            u_int16_t domain;
            u_int8_t  bus;
            u_int8_t  dev;
            u_int8_t  func;

            u_int16_t dev_id;
            u_int16_t vend_id;
            u_int32_t class_id;
            u_int16_t subsys_id;
            u_int16_t subsys_vend_id;

            char      cr_dev[512];
            char      conf_dev[512];
            char**    net_devs;      // Null terminated array
            char**    ib_devs;       // Null terminated array
            char      numa_node[4096];     //
        } pci;

        struct {
            u_int32_t mtusb_serial;
            u_int32_t TBD;
        } usb;

        struct {
            u_int32_t TBD;
        } ib;

        struct {
            u_int32_t TBD;
        } remote;
    };
} dev_info;

typedef struct icmd_params_t {
	int icmd_opened;
	int took_semaphore;
	int ctrl_addr;
	int cmd_addr;
	u_int32_t max_cmd_size;
	int semaphore_addr;
	int static_cfg_not_done_addr;
	int static_cfg_not_done_offs;

}icmd_params;

typedef struct ctx_params_t {
	void *fw_cmd_context;
	void *fw_cmd_func;
}ctx_params;

typedef struct io_region_t {
    unsigned int start;
    unsigned int end;
}io_region;

typedef struct tools_hcr_params_t {
    int supp_cr_mbox; // 1: mbox supported , -1: mbox not supported
}tools_hcr_params;

typedef struct access_reg_params_t {
    int max_reg_size;
}access_reg_params;


/*  All fields in follow structure are not supposed to be used */
/*  or modified by user programs. Except i2c_slave that may be */
/*  modified before each access to target I2C slave address */
typedef struct mfile_t {
    MType         tp;           /*  type of driver */
    MType         res_tp;       /*  Will be used with HCR if need */
    DType         dtype;        /*  target device to access to */
    DType         itype;        /*  interface device to access via */
    int           is_i2cm;      /*  use device as I2C master */
    int           is_vm;        /*  if the machine is VM    */
    int           cr_access;    /* If cr access is allowed in MLNXOS devices */
    unsigned char i2c_slave;
    io_region*    iorw_regions;   /* For LPC devices */
    int           regions_num;
#ifdef __WIN__
    MT_ulong_ptr_t fd;
#else
    int           fd;
    int           res_fd;       /*  Will be used with HCR if need*/
#endif
    int           sock;         /*  in not -1 - remote interface */
    void         *ptr;
    unsigned int  map_size;
    unsigned long long start_idx;
    /************************ FPGA DDR3 *********************************/
    void         *ddr3_ptr;
    unsigned long long ddr3_start_idx;
    unsigned int  ddr3_map_size;
    /********************************************************************/
    MIB_Private   mib;          /*  Data for IB interface (if relevant) */
	void         *ctx;
    unsigned int  i2c_RESERVED; /*  Reserved for internal usage (i2c internal) */
    enum Mdevs_t  flags;
    u_int32_t     connectx_wa_slot; /* apply connectx cr write workaround */
    int           connectx_wa_last_op_write;
    u_int32_t     connectx_wa_stat;
    u_int64_t     connectx_wa_max_retries;
    u_int64_t     connectx_wa_num_of_writes;
    u_int64_t     connectx_wa_num_of_retry_writes;
    int           server_ver_major;
    int           server_ver_minor;
    unsigned int  proto_type;
    dev_info*     dinfo;

    //for ICMD access
    icmd_params icmd;
    // for UEFI
    ctx_params context;
    // for new pci capability
    int old_mst;
    int vsec_supp;
    int address_space;
    int multifunction;
    // for tools HCR access
    tools_hcr_params hcr_params;
    // for sending access registers
    access_reg_params acc_reg_params;
    //
} mfile;

#define SLV_ADDRS_NUM 128

#ifdef __WIN__
#define FromHandle(h)   ((MT_ulong_ptr_t)(h))
#define ToHandle(h)     ((HANDLE)(h))
#else
#define FromHandle(h)   ((int)(h))
#define ToHandle(h)     ((HANDLE)(h))
#endif

/*
 * Get list of MST (Mellanox Software Tools) devices.
 * Put all device names as null-terminated strings to buf.
 *
 * Return number of devices found or -1 if buf overflow
 */
MTCR_API int mdevices(char *buf, int len, int mask);

/*
 * Get list of MST (Mellanox Software Tools) devices info records.
 * Return a dynamic allocated array of dev_info records.
 * len will be updated to hold the array length
 *
 */
MTCR_API dev_info* mdevices_info(int mask, int* len);

/*
 * Destroy the array of dev_info recored obtained by mdevices_info\() function
 *
 */
MTCR_API void mdevices_info_destroy(dev_info* dev_info, int len);

/*
 * Open Mellanox Software tools (mst) driver.
 * Return valid mfile ptr or 0 on failure
 */
MTCR_API mfile *mopend(const char *name, DType dtype);

/*
 * Open Mellanox Software tools (mst) driver.
 * Return valid mfile ptr or 0 on failure
 */
MTCR_API mfile *mopen_adv(const char *name, MType mtype);

/*
 * Open Mellanox Software tools (mst) driver. Device type=InfiniHost MType=MST_DEFAULT
 * Return valid mfile ptr or 0 on failure
 */
MTCR_API mfile *mopen(const char *name);

/*
 * Open Mellanox Software tools with uefi driver
 * Return valid mfile ptr or 0 on failure
 * NOTE: mfile will not conatin device data just context and uefi access function
 * supports only maccess_reg() function.
 */
MTCR_API mfile *mopen_fw_ctx(void* fw_cmd_context, void* fw_cmd_func, void* extra_info);

/*
 * Close Mellanox driver
 * req. descriptor
 */
MTCR_API int mclose(mfile *mf);

/*
 * Accelerate device if possible.
 * When device is I2C master - overclock it
 */
MTCR_API void maccelerate(mfile *mf);

/*
 * Restore normal settings, if device was accelerated.
 */
MTCR_API void mrestore(mfile *mf);

/*
 * Read 4 bytes, return number of succ. read bytes or -1 on failure
 */
MTCR_API int mread4(mfile *mf, unsigned int offset, u_int32_t *value);

/*
 * Write 4 bytes, return number of succ. written bytes or -1 on failure
 */
MTCR_API int mwrite4(mfile *mf, unsigned int offset, u_int32_t value);

/*
 * Read a block of dwords, return number of succ. read bytes or -1 on failure
 * Works for any interface, but can be faster for interfaces where bursts
 * are supported (MTUSB, IB).
 * Data retrns in the same endianess of mread4/mwrite4
 */
MTCR_API int mread4_block (mfile *mf, unsigned int offset, u_int32_t* data, int byte_len);
MTCR_API int mwrite4_block(mfile *mf, unsigned int offset, u_int32_t* data, int byte_len);

/* read buffer as is without changing endians */
MTCR_API int mread_buffer(mfile *mf, unsigned int offset, u_int8_t* data, int byte_len);

/* Write buffer as is without changing endians */

MTCR_API int mwrite_buffer(mfile *mf, unsigned int offset, u_int8_t* data, int byte_len);

/*
 * Read up to 64 bytes, return number of succ. read bytes or -1 on failure
 */
MTCR_API int mread64(mfile *mf, unsigned int offset, void *data, int length);

/*
 * Write up to 64 bytes, return number of succ. written bytes or -1 on failure
 */
MTCR_API int mwrite64(mfile *mf, unsigned int offset, void *data, int length);

/*
 * Read up to 64 bytes, return number of succ. read bytes or -1 on failure
 */
MTCR_API int mread_i2cblock(mfile *mf, unsigned char i2c_slave, u_int8_t addr_width,
                            unsigned int offset, void *data, int length);

/*
 * Write up to 64 bytes, return number of succ. written bytes or -1 on failure
 */
MTCR_API int mwrite_i2cblock(mfile *mf, unsigned char i2c_slave, u_int8_t addr_width,
                             unsigned int offset, void *data, int length);

/*
 * Set a new value for i2c_slave
 * Return previous value
 */
MTCR_API unsigned char mset_i2c_slave(mfile *mf, unsigned char new_i2c_slave);
MTCR_API int mget_i2c_slave(mfile *mf, unsigned char *new_i2c_slave_p);


MTCR_API int mset_i2c_addr_width(mfile *mf, u_int8_t  addr_width);
MTCR_API int mget_i2c_addr_width(mfile *mf, u_int8_t* addr_width);

MTCR_API int mget_mdevs_flags(mfile *mf, u_int32_t *devs_flags);
MTCR_API int mget_mdevs_type(mfile *mf, u_int32_t *mtype);
/*
 * Software reset the device.
 * Return 0 on success, <0 on failure.
 * Currently supported for IB device only.
 * Mellanox switch devices support this feature.
 * HCAs may not support this feature.
 */
MTCR_API int msw_reset(mfile *mf);

/*
 * reset the device.
 * Return 0 on success, <0 on failure.
 * Curently supported on 5th Generation HCAs.
 */
MTCR_API int mhca_reset(mfile *mf);

MTCR_API int mi2c_detect (mfile *mf, u_int8_t slv_arr[SLV_ADDRS_NUM]);

#define MTCR_MFT_2_7_0

typedef enum {
    RA_MFPA=0x9010,
    RA_MFBA=0x9011,
    RA_MFBE=0x9012,
} reg_access_t;

MTCR_API int maccess_reg_mad(mfile *mf, u_int8_t *data);

MTCR_API int maccess_reg_cmdif(mfile *mf, reg_access_t reg_access, void *reg_data, u_int32_t cmd_type);

MTCR_API int maccess_reg(mfile     *mf,
                         u_int16_t  reg_id,
                         maccess_reg_method_t reg_method,
                         void*      reg_data,
                         u_int32_t  reg_size,
                         u_int32_t	r_size_reg, // used when sending via icmd interface (how much data should be read back to the user)
                         u_int32_t	w_size_reg, // used when sending via icmd interface (how much data should be written to the scratchpad)
                         	 	 	 	 	 	// if you dont know what you are doing then r_size_reg = w_size_reg = your_register_size
                         int       *reg_status);

/**
 * Handles the send command procedure.
 * for completeness, but calling it is strongly advised against.
 * @param[in] dev   A pointer to a device context, previously
 *                  obtained by a call to <tt>gcif_open</tt>.
 * @return          One of the GCIF_STATUS_* values, or a raw
 *                  status value (as indicated in cr-space).
 * NOTE: when calling this function the caller needs to make
 *  	sure device supports icmd.
 **/
MTCR_API int icmd_send_command(mfile    *mf,
                      int     opcode,
                      void*   data,
                      int     data_size,
                      int     skip_write);

/**
 * Clear the Tools-HCR semaphore. Use this when an application
 * that uses this library is not terminated cleanly, leaving the
 * semaphore in a locked state.
 * @param[in] dev   A pointer to a device context, previously
 *                  obtained by a call to <tt>gcif_open</tt>.
 * @return          One of the GCIF_STATUS_* values, or a raw
 *                  status value (as indicated in cr-space).
 * NOTE: when calling this function the caller needs to make
 *  	sure device supports icmd.
 **/
MTCR_API int icmd_clear_semaphore(mfile *mf);

/*
 * send an inline command to the tools HCR
 * limitations:
 * command should not use mailbox
 * NOTE: when calling this function caller needs to make
 *       sure device support tools HCR
 */
MTCR_API int tools_cmdif_send_inline_cmd(mfile* mf,
                     u_int64_t in_param,
                     u_int64_t* out_param,
                     u_int32_t input_modifier,
                     u_int16_t opcode,
                     u_int8_t  opcode_modifier);

/*
 * send a mailbox command to the tools HCR
 * limitations:
 * i.e write data to mailbox execute command (op = opcode op_modifier= opcode_modifier) and read data back from mailbox
 * data_offs_in_mbox: offset(in bytes) to read and write data to and from mailbox should be quad word alligned.
 *  * NOTE: when calling this function caller needs to make
 *       sure device support tools HCR
 */
MTCR_API int tools_cmdif_send_mbox_command(mfile* mf,
                                           u_int32_t input_modifier,
                                           u_int16_t opcode,
                                           u_int8_t  opcode_modifier,
                                           int data_offs_in_mbox,
                                           void* data,
                                           int data_size,
                                           int skip_write);

MTCR_API int tools_cmdif_unlock_semaphore(mfile *mf);

/*
 * returns the maximal allowed register size (in bytes)
 * according to the FW access method or -1 if no restriction applicable
 *
 */
MTCR_API int mget_max_reg_size(mfile *mf);
/*
 * translate virtual address to physical address
 * return physical address on success, or 0 on error
 */
MTCR_API unsigned long mvtop (mfile *mf, void *va);

MTCR_API const char* m_err2str(MError status);

MTCR_API int mvpd_read4(mfile *mf, unsigned int offset, u_int8_t value[4]);

MTCR_API int mvpd_write4(mfile *mf, unsigned int offset, u_int8_t value[4]);

MTCR_API int mib_smp_set(mfile* mf, u_int8_t* data, u_int16_t attr_id, u_int32_t attr_mod);
MTCR_API int mib_smp_get(mfile* mf, u_int8_t* data, u_int16_t attr_id, u_int32_t attr_mod);

MTCR_API int mset_cr_access(mfile* mf, int access);

MTCR_API int mget_vsec_supp(mfile* mf);

MTCR_API int mget_addr_space(mfile* mf);
MTCR_API int mset_addr_space(mfile* mf, int space);

#ifdef __cplusplus
}
#endif


#endif
